<script setup lang="ts">
import { h, onMounted, ref, toRaw } from 'vue';

import { Page, type VbenFormProps } from '@vben/common-ui';

import { Space } from 'ant-design-vue';
import dayjs from 'dayjs';

import { useVbenVxeGrid, type VxeGridProps, vxeSortEvent } from '#/adapter/vxe-table';
import { insp } from '#/api/eims/report';

import { columns as dynamicColumns, querySchema } from './data';
// 数据是否初始化
const initFlag = ref(false);
const baseColumns: VxeGridProps['columns'] = [
  {
    title: '设备名称',
    field: 'equName',
    minWidth: 120,
    fixed: 'left'
  },
  {
    title: '资产编号 - 日期',
    field: 'assetNo',
    minWidth: 120,
    fixed: 'left'
  }
];
initColumns();

const formOptions: VbenFormProps = {
  commonConfig: {
    labelWidth: 80,
    componentProps: {
      allowClear: true
    }
  },
  schema: querySchema(),
  wrapperClass: 'grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4',
  handleReset: async () => {
    // eslint-disable-next-line no-use-before-define
    const { formApi, reload } = tableApi;
    await formApi.resetForm();
    const formValues = formApi.form.values;
    formApi.setLatestSubmissionValues(formValues);
    await reload(formValues);
    initFlag.value = false;
    initColumns(dayjs().format('YYYY-MM'), true);
  },
  // 日期选择格式化
  fieldMappingTime: [['changeDate', ['params[beginTime]', 'params[endTime]'], ['YYYY-MM-DD 00:00:00', 'YYYY-MM-DD 23:59:59']]]
};

const gridOptions: VxeGridProps = {
  checkboxConfig: {
    // 高亮
    highlight: true,
    // 翻页时保留选中状态
    reserve: true
    // 点击行选中
    // trigger: 'row'
  },
  border: true,
  size: 'mini',
  columns: dynamicColumns.value,
  height: 'auto',
  keepSource: true,
  pagerConfig: {
    // 默认条数
    pageSize: 20,
    // 分页可选条数
    pageSizes: [10, 20, 30, 50, 100, 200, 300, 500]
  },
  proxyConfig: {
    ajax: {
      query: async ({ page }, formValues = {}) => {
        // TODO 暂时解决动态修改表头后参数丢失问题
        // eslint-disable-next-line no-use-before-define
        const p = await tableApi.formApi.getValues();
        const params = toRaw(p);

        return await insp({
          pageNum: page.currentPage,
          pageSize: page.pageSize,
          ...params
        });
      }
    }
  },
  rowConfig: {
    isHover: true,
    keyField: 'inspReport'
  },
  sortConfig: {
    // 远程排序
    remote: true,
    // 支持多字段排序 默认关闭
    multiple: true
  },
  id: 'eims-report-index'
};

const [BasicTable, tableApi] = useVbenVxeGrid({
  formOptions,
  gridOptions,
  gridEvents: {
    sortChange: (sortParams) => vxeSortEvent(tableApi, sortParams)
  }
});

onMounted(async () => {
  initListener();
  // await setupEquSelect();
});
const selMonth = ref('');

function initListener() {
  tableApi.formApi.updateSchema([
    {
      component: 'DatePicker',
      componentProps: {
        onChange: (sMonth: string) => {
          selMonth.value = sMonth;
          dynamicColumns.value = [];
          initFlag.value = false;
          initColumns(sMonth, true);
        }
      },
      fieldName: 'selectMonth'
    }
  ]);
}

function initColumns(selectMonth: string = dayjs().format('YYYY-MM'), isUpdate: boolean = false) {
  if (initFlag.value) {
    return false;
  }
  const dayNumbers = getDayNumbers(selectMonth);
  const newColumns: VxeGridProps['columns'] = dayNumbers.map((day: number) => ({
    title: `${day}`,
    field: `day_${day}`,
    width: 40,
    align: 'center',
    slots: {
      default: ({ row }) => {
        const value = row[`day_${day}`];
        let className = '';

        switch (value) {
          case '已完成': {
            className = 'dot-green';
            break;
          }
          case '待验证': {
            className = 'dot-orange';
            break;
          }
          case '未点检': {
            className = 'dot-gray';
            break;
          }
          default: {
            return '-';
          }
        }

        return h('span', { class: className });
      }
    }
    // formatter: ({ row }: { row: any }) => {
    //   return row[`day_${day}`] || '-';
    // }
  }));
  dynamicColumns.value = [...(baseColumns || []), ...newColumns];
  initFlag.value = true;
  if (isUpdate) {
    tableApi.setGridOptions({
      ...gridOptions,
      columns: dynamicColumns.value
    });
    tableApi.query();
  }
}

function getDayNumbers(yearMonth: string): number[] {
  const days = dayjs(yearMonth).daysInMonth();
  return Array.from({ length: days }).map((_, i) => i + 1);
}
</script>

<template>
  <Page :auto-content-height="true">
    <div class="flex h-full gap-[8px]">
      <BasicTable class="flex-1 overflow-hidden" table-title="试产列表">
        <template #toolbar-tools>
          <Space>
            <span class="ml-4 mr-2">-</span>未生成计划 <span class="dot-gray ml-4"></span>未点检 <span class="dot-orange ml-4"></span>待验证
            <span class="dot-green ml-4"></span>已点检
          </Space>
        </template>

        <template #insp="{ row }">
          <Space>
            <span>{{ row.equName }}</span>
          </Space>
        </template>
      </BasicTable>
    </div>
  </Page>
</template>
<style lang="less" scoped>
:deep(.dot-green) {
  display: inline-block;
  width: 10px;
  height: 10px;
  border-radius: 50%;
  background-color: #52c41a;
}

:deep(.dot-orange) {
  display: inline-block;
  width: 10px;
  height: 10px;
  border-radius: 50%;
  background-color: #faad14;
}

:deep(.dot-gray) {
  display: inline-block;
  width: 10px;
  height: 10px;
  border-radius: 50%;
  background-color: #d9d9d9;
}
</style>
